#!/usr/bin/env python2
import subprocess
import json


def call_process(cmd):
    p = subprocess.Popen(
        cmd.split(),
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)
    output = p.communicate()
    return output


def ovsctl_json(cmd):
    cmd = "ovs-vsctl -f json " + cmd
    result = call_process(cmd)
    headings = json.loads(result[0])["headings"]
    datas = json.loads(result[0])["data"]

    uuid_index = headings.index("_uuid")

    obj = {}

    for data in datas:
        obj_data = {}
        for d, h in zip(data, headings):
            if isinstance(d, list):
                _type, value = d[0], d[1]
                if _type == "uuid":
                    obj_data[h] = [d]
                elif _type == "map":
                    obj_data[h] = {v[0]: v[1] for v in value}
                elif _type == "set":
                    obj_data[h] = value
            else:
                obj_data[h] = d
        obj[data[uuid_index][1]] = obj_data
    return obj


def gather():
    bridges = ovsctl_json("list bridge")
    ports = ovsctl_json("list port")
    interfaces = ovsctl_json("list interface")

    for bridge_uuid, bridge in bridges.iteritems():
        for bridge_port in bridge['ports']:
            port = ports[bridge_port[1]]
            interface = interfaces[port['interfaces'][0][1]]

            label_obj = {
                "bridge": bridge.get('name'),
                "port": port.get('name'),
                "interface": interface.get('name'),
                "type": interface.get('type'),
                "peer": interface.get('options', {}).get('peer'),
            }

            labels = ','.join(["{}={}".format(key, value)
                               for key, value in label_obj.iteritems()
                               if not (value is None or len(value) == 0)])

            if interface['link_state'] == "up":
                status = 1
                if len(interface['error']) > 0:
                    status = 2
            else:
                status = 0

            print "ovs_bridge,{} status={}".format(labels, status)


if __name__ == "__main__":
    try:
        gather()
        print "ovs_bridge check=1"
    except Exception:
        print "ovs_bridge check=0"
